- test:
    timeout:
      minutes: {{ timeout }}
    interactive:
    - name: virtio_net
      prompts: ["{{ '", "'.join(prompts)}}"]
      script:
      - command: "dmesg"
        name: virtio_net
        successes:
        - message: 'smc91x.{0,4}: not found'

