- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: https://github.com/roxell/test-definitions.git
      lava-signal: kmsg
      from: git
      branch: mmtests
      path: automated/linux/mmtests/mmtests.yaml
      parameters:
        SKIP_INSTALL: 'true'
        MMTESTS_CONFIG_FILE: '{{ configfile }}'
        MMTEST_ITERATIONS: '{{ parameters.get("ITERATIONS", iterations) }}'
        TEST_DIR: '{{ parameters.get("MMTESTS_PATH", "/mmtests") }}'
        FULL_ARCHIVE: '{{ parameters.get("FULL_ARCHIVE", "False")|lower }}'
        SHARED_DIR: 'true'
      name: {{ name }}

